/*
 * AbstractInstrumenter.java
 *
 * Created on August 3, 2001, 9:27 AM
 *
 * See LICENSE file for license conditions.
 */

package residue;

import java.util.*;
import residue.events.*;

/**
 * Abstract super class for the classes that perform
 * instrumentation in one form or another.  Provides
 * the hooks to use the events package.
 *
 * @author  chowells
 * @version 
 */
public abstract class AbstractInstrumenter
{
	private Set listeners = new HashSet();
	
	/**
	 * This this method registers an InstrumentationEventListener
	 * with the class, which events generated by the other methods
	 * in this class are sent to.
	 */
	public void addInstrumentationEventListener(InstrumentationEventListener e)
	{
		listeners.add(e);
	} // addInstrumentationEventListener
	
	/**
	 * Creates a new InstrumentationEvent, with the given fields,
	 * and sends it to this class's listeners' instrumentationStarted
	 * method.
	 */
	protected void instrumentationStarted(String classname, int remaining)
	{
		InstrumentationEvent e = new InstrumentationEvent(classname, remaining);
		Iterator it = listeners.iterator();
		while (it.hasNext())
		{
			InstrumentationEventListener il = (InstrumentationEventListener)it.next();
			il.instrumentationStarted(e);
		} // while
	} // instrumentationStarted
	
	/**
	 * Creates a new InstrumentationEvent, with the given fields,
	 * and sends it to this class's listeners' instrumentationFinished
	 * method.
	 */
	protected void instrumentationFinished(String classname, int remaining)
	{
		InstrumentationEvent e = new InstrumentationEvent(classname, remaining);
		Iterator it = listeners.iterator();
		while (it.hasNext())
		{
			InstrumentationEventListener il = (InstrumentationEventListener)it.next();
			il.instrumentationFinished(e);
		} // while
	} // instrumentationFinished
	
	/**
	 * Creates a new InstrumentationEvent, with the given fields,
	 * and sends it to this class's listeners' classStarted
	 * method.
	 */
	protected void classStarted(String classname, int remaining)
	{
		InstrumentationEvent e = new InstrumentationEvent(classname, remaining);
		Iterator it = listeners.iterator();
		while (it.hasNext())
		{
			InstrumentationEventListener il = (InstrumentationEventListener)it.next();
			il.classStarted(e);
		} // while
	} // classStarted
	
} // class AbstractInstrumenter
