/*
 * ListHits.java
 *
 * Created on June 22, 2001, 4:19 AM
 *
 * See LICENSE file for license conditions.
 */

package residue;

import java.io.*;
import java.util.*;
import residue.runtime.Monitor;

import residue.tables.*;

/**
 *
 * @author  chowells
 * @version 
 */
public class ListHits
{
	/**
	 * Shows what lines in what files have been hit, or if
	 * given the value "-m" in args[0], shows what instrumented
	 * lines have been missed.
	 */
	public static void main(String [] args) throws IOException
	{
		if (args.length < 1)
		{
			System.out.println("Please give the path to the correct config file");
			System.out.println("on the command line.");
			System.exit(1);
		}
		
		Map m;
		String pre;
		if (args[0].equalsIgnoreCase("-m"))
		{
			setConfigFile(args[1]);
			m = getMisses();
			pre = "Miss";
		}
		else
		{
			setConfigFile(args[0]);
			m = getHits();
			pre = "Hit";
		}
		
		Iterator it = m.keySet().iterator();
		while (it.hasNext())
		{
			String s = (String)it.next();
			Set l = (Set)m.get(s);
			Iterator it2 = l.iterator();
			while (it2.hasNext())
			{
				System.out.println(pre + " in source file: " + s + ", on line " + it2.next());
			}
		}
		
	} // main
	
	/**
	 * This method sets the config file used to get the table locations.
	 */
	public static synchronized void setConfigFile(String filename) throws IOException
	{
		gotten = false;
		Monitor.getNames(filename);
	} // setConfigFile
	
	/**
	 * returns a map from source names -> Set of hit lines (Integer)
	 */
	public static synchronized Map getHits() throws IOException
	{
		if (!gotten) get();
		return hits;
	} // getHits
	
	/**
	 * returns a map from source names -> Set of missed lines (Integer)
	 */
	public static synchronized Map getMisses() throws IOException
	{
		if (!gotten) get();
		return misses;
	} // getMisses
	
	/**
	 * Retrieves the LineTable most recently used in this class.
	 */
	public static synchronized Table getTable() throws IOException
	{
		if (!gotten) get();
		return table;
	} // getTable
	
	/**
	 * Refreshes the contents of the maps returned in getMisses()
	 * and getHits()
	 */
	public static synchronized void update() throws IOException
	{
		get();
	} // update
	
	private static Map hits;
	private static Map misses;
	private static boolean gotten;
	private static Table table;
	
	/**
	 * sets the internal maps returned by the get methods
	 */
	private static void get() throws IOException
	{
		gotten = true;
		
		hits = new HashMap();
		misses = new HashMap();
		
		table = Table.getTable(Monitor.getLineTableName());
		String htname = Monitor.getHitTableName();
		
		Iterator it = table.iterator();
		while (it.hasNext())
		{
			TableEntry te = (TableEntry)it.next();
			
			if (te.wasExecuted())
			{
				// hit
				if (!hits.containsKey(te.getSourceName()))
					hits.put(te.getSourceName(), new TreeSet());
				
				Set h = (Set)hits.get(te.getSourceName());
				h.addAll(te.getLineSet());
			}
			else
			{
				// missed
				if (!misses.containsKey(te.getSourceName()))
					misses.put(te.getSourceName(), new TreeSet());
				
				Set m = (Set)misses.get(te.getSourceName());
				m.addAll(te.getLineSet());
			}
		} // while
		
	} // get
	
} // class ListHits
