/*
 * Main.java
 *
 * Created on August 30, 2001, 6:30 AM
 *
 * See LICENSE file for license conditions.
 */

package residue;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Rectangle;
import java.awt.Cursor;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import residue.events.*;
import residue.tables.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import residue.runtime.Monitor;
import javax.swing.filechooser.FileFilter;

/**
 * The main GUI for Gretel.  A large section of the code for
 * laying out the GUI was generated using the Netbeans 3.2 IDE.
 *
 * main also contains code for dispatching to the other utils
 * based on command line args.
 *
 * @author  chowells
 */
public class Main extends javax.swing.JFrame
{
	private HelpFrame hf;
	
	private JFileChooser classChooser = new JFileChooser();
	private JFileChooser configChooser = new JFileChooser();
	private JFileChooser dirChooser = new JFileChooser();
	
	private DefaultListModel insModel = new DefaultListModel();
	private DefaultListModel reinsModel = new DefaultListModel();
	private DefaultListModel remModel = new DefaultListModel();
	
	// class (String) -> source (String) map
	private Map sources = new HashMap();
	
	private String reinsConfig = "";
	
	// List of SourceViewers created
	private List viewers = new ArrayList();
	
	// enums used for currentInsSetting
	private static final int NOREDUCE = 0;
	private static final int CONSERVATIVE = 1;
	private static final int AGGRESSIVE = 2;
	
	private int currentInsSetting = AGGRESSIVE;
	
	private Rectangle correctBounds;
	
	// properties stored in the .gretelrc file
	private static final String LEFT_BOUND_PROPERTY = "gui.leftBound";
	private static final String TOP_BOUND_PROPERTY = "gui.topBound";
	private static final String WIDTH_PROPERTY = "gui.width";
	private static final String HEIGHT_PROPERTY = "gui.height";
	private static final String CURRENT_TAB_PROPERTY = "gui.currentTab";
	private static final String CURRENT_CONFIG_PROPERTY = "gui.currentConfig";
	
	// strings used for tooltips
	private static final String ins0 = "Please choose files and directory";
	private static final String ins1 = "Please choose a directory";
	private static final String ins2 = "Please choose files";
	private static final String ins3 = "Click here to instrument";
	
	private static final String reins0 = "Please choose config file";
	private static final String reins1 = "Click here to reinstrument";
	
	private static final String rem0 = "Please choose files";
	private static final String rem1 = "Click here to remove instrumentation";
	
	public Main()
	{
		initChoosers();
		initComponents();
		initLists();
		initEnablers();
		pack();
		loadSettings();
	} // Main
	
	private void loadSettings()
	{
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		
		int top = (d.height - getHeight()) / 2;
		int left = (d.width - getWidth()) / 2;
		int w = getWidth();
		int h = getHeight();
		
		String propname = System.getProperty("user.home") + File.separator + ".gretelrc";
		
		Properties p = new Properties();
		try
		{
			InputStream is = new FileInputStream(propname);
			p.load(is);
			
			is.close();
		}
		catch (IOException e) {}
		
		try { top = Integer.parseInt(p.getProperty(TOP_BOUND_PROPERTY)); }
		catch (Exception e) {}
		
		try { left = Integer.parseInt(p.getProperty(LEFT_BOUND_PROPERTY)); }
		catch (Exception e) {}
		
		try { w = Integer.parseInt(p.getProperty(WIDTH_PROPERTY)); }
		catch (Exception e) {}
		
		try { h = Integer.parseInt(p.getProperty(HEIGHT_PROPERTY)); }
		catch (Exception e) {}
		
		setBounds(left, top, w, h);
		correctBounds = new Rectangle(left, top, w, h);
		
		// load other settings.
		
		try
		{
			jTabbedPane1.setSelectedIndex(Integer.decode(
				p.getProperty(CURRENT_TAB_PROPERTY)).intValue());
		}
		catch (Exception e) {}
		
		String cfg = p.getProperty(CURRENT_CONFIG_PROPERTY);
		if (cfg != null) reinsConfigField.setText(cfg);
		
	} // loadSettings
	
	
	/**
	 * Sets up listeners to control enabling/disabling buttons on the
	 * Instrumentation panel.
	 */
	private void initInsEnablers()
	{
		insModel.addListDataListener(new ListDataListener()
		{
			public void contentsChanged(ListDataEvent e)
			{
				if (insModel.size() == 0)
				{
					insClearButton.setEnabled(false);
					insClearButton.repaint();
				}
				else
				{
					insClearButton.setEnabled(true);
					insClearButton.repaint();
				}
				
				updateInsButton();
				insList.getSelectionModel().clearSelection();
			} // contentsChanged
			
			public void intervalAdded(ListDataEvent e)
			{
				insClearButton.setEnabled(true);
				insClearButton.repaint();
				
				updateInsButton();
				insList.getSelectionModel().clearSelection();
			} // intervalAdded
			
			public void intervalRemoved(ListDataEvent e)
			{
				if (insModel.size() == 0)
				{
					insClearButton.setEnabled(false);
					insClearButton.repaint();
				}
				
				updateInsButton();
				insList.getSelectionModel().clearSelection();
			} // intervalRemoved
		}); // anonymous ListDataListener
		
		insList.getSelectionModel().addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e)
			{
				if (insList.getSelectedIndices().length > 0)
				{
					insRemoveButton.setEnabled(true);
					insRemoveButton.repaint();
				}
				else
				{
					insRemoveButton.setEnabled(false);
					insRemoveButton.repaint();
				}
			} // valueChanged
		}); // anonymous ListSelectionListener
		
		insDirectoryField.getDocument().addDocumentListener(new DocumentListener()
		{
			public void changedUpdate(DocumentEvent e)
			{
				updateInsButton();
			} // changedUpdate
			
			public void insertUpdate(DocumentEvent e)
			{
				updateInsButton();
			} // insertUpdate
			
			public void removeUpdate(DocumentEvent e)
			{
				updateInsButton();
			} // removeUpdate
		}); // anonymous DocumentListener
		
	} // initInsEnablers
	
	/**
	 * Helper button to centralize all the logic for
	 * whether the Instrument Button should be enabled
	 * or not.
	 */
	private void updateInsButton()
	{
		File f = new File(insDirectoryField.getText());
		
		if (f.exists() && f.isDirectory() && insModel.size() > 0)
		{
			insButton.setToolTipText(ins3);
			
			insButton.setEnabled(true);
			insButton.repaint();
		}
		else
		{
			if (f.exists() && f.isDirectory()) insButton.setToolTipText(ins2);
			else if (insModel.size() > 0) insButton.setToolTipText(ins1);
			else insButton.setToolTipText(ins0);
			
			insButton.setEnabled(false);
			insButton.repaint();
		}
	} // updateInsButton
	
	/**
	 * Set up listeners to enable/disable buttons on the
	 * Reinstrument panel.
	 */
	private void initReinsEnablers()
	{
		reinsConfigField.getDocument().addDocumentListener(new DocumentListener()
		{
			public void changedUpdate(DocumentEvent e)
			{
				// This probably will never be called, but it's here
				// just for the sake of completeness.
				updateReinsList();
			} // changedUpdate
			
			public void insertUpdate(DocumentEvent e)
			{
				updateReinsList();
			} // insertUpdate
			
			public void removeUpdate(DocumentEvent e)
			{
				updateReinsList();
			} // removeUpdate
		}); // anonymous DocumentListener
		
		reinsList.getSelectionModel().addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e)
			{
				viewButton.setEnabled(true);
				viewButton.repaint();
			} // valueChanged
		}); // anonymous ListSelectionListener
		
	} // initReinsEnablers
	
	/**
	 * set up listeners to enable/disable buttons on the
	 * Remover panel.
	 */
	private void initRemEnablers()
	{
		remModel.addListDataListener(new ListDataListener()
		{
			public void contentsChanged(ListDataEvent e)
			{
				if (remModel.size() == 0)
				{
					remClearButton.setEnabled(false);
					remClearButton.repaint();
					remButton.setToolTipText(rem0);
					remButton.setEnabled(false);
					remButton.repaint();
				}
				else
				{
					remClearButton.setEnabled(true);
					remClearButton.repaint();
					remButton.setToolTipText(rem1);
					remButton.setEnabled(true);
					remButton.repaint();
				}
				
				remList.getSelectionModel().clearSelection();
			} // contentsChanged
			
			public void intervalAdded(ListDataEvent e)
			{
				remClearButton.setEnabled(true);
				remClearButton.repaint();
				remButton.setToolTipText(rem1);
				remButton.setEnabled(true);
				remButton.repaint();
				
				remList.getSelectionModel().clearSelection();
			} // intervalAdded
			
			public void intervalRemoved(ListDataEvent e)
			{
				if (remModel.size() == 0)
				{
					remClearButton.setEnabled(false);
					remClearButton.repaint();
					remButton.setToolTipText(rem0);
					remButton.setEnabled(false);
					remButton.repaint();
				}
				
				remList.getSelectionModel().clearSelection();
			} // intervalRemoved
		}); // anonymous ListDataListener
		
		remList.getSelectionModel().addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e)
			{
				if (remList.getSelectedIndices().length > 0)
				{
					remRemoveButton.setEnabled(true);
					remRemoveButton.repaint();
				}
				else
				{
					remRemoveButton.setEnabled(false);
					remRemoveButton.repaint();
				}
			} // valueChanged
		}); // anonymous ListSelectionListener
		
	} // initRemEnablers
	
	/**
	 * Sets up listeners to enable/disable buttons.
	 */
	private void initEnablers()
	{
		initInsEnablers();
		initReinsEnablers();
		initRemEnablers();
	} // initEnablers
	
	/**
	 * Sets up the JLists, connecting them to their
	 * ListModels, and setting up double clicking
	 * on the Reinstrumentation JList to start an
	 * appropriate sourceviewer.
	 */
	private void initLists()
	{
		insList.setModel(insModel);
		
		reinsList.setModel(reinsModel);
		reinsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		remList.setModel(remModel);
	} // initListModels
	
	/**
	 * Sets up the various JFileChoosers with their FileFilters
	 * and sets them start in the current directory.
	 */
	private void initChoosers()
	{
		File currdir = new File(System.getProperty("user.dir"));
		
		classChooser.setCurrentDirectory(currdir);
		classChooser.setMultiSelectionEnabled(true);
		classChooser.setAcceptAllFileFilterUsed(false);
		classChooser.setFileFilter(new FileFilter()
		{
			public boolean accept(File f)
			{
				if (f.isDirectory()) return true;
				return f.getName().endsWith(".class");
			} // accept
			
			public String getDescription()
			{
				return ".class files";
			} // getDescription
		}); // anonymous FileFilter
		classChooser.addPropertyChangeListener(
			JFileChooser.DIRECTORY_CHANGED_PROPERTY, new PropertyChangeListener()
		{
			public void propertyChange(PropertyChangeEvent e)
			{
				updateChoosers((File)e.getNewValue());
			} // propertyChange
		}); // anonymous PropertyChangeListener
		
		configChooser.setCurrentDirectory(currdir);
		configChooser.setAcceptAllFileFilterUsed(false);
		configChooser.setFileFilter(new FileFilter()
		{
			public boolean accept(File f)
			{
				if (f.isDirectory()) return true;
				return f.getName().endsWith(".gretel");
			} // accept
			
			public String getDescription()
			{
				return "Gretel configuration files";
			} // getDescription
		}); // anonymous FileFilter
		configChooser.addPropertyChangeListener(
			JFileChooser.DIRECTORY_CHANGED_PROPERTY, new PropertyChangeListener()
		{
			public void propertyChange(PropertyChangeEvent e)
			{
				updateChoosers((File)e.getNewValue());
			} // propertyChange
		}); // anonymous PropertyChangeListener
		
		dirChooser.setCurrentDirectory(currdir);
		dirChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		dirChooser.setAcceptAllFileFilterUsed(false);
		dirChooser.setFileFilter(new FileFilter()
		{
			public boolean accept(File f)
			{
				return f.isDirectory();
			} // accept
			
			public String getDescription()
			{
				return "Directories";
			} // getDescription
		}); // anonymous FileFilter
		dirChooser.addPropertyChangeListener(
			JFileChooser.DIRECTORY_CHANGED_PROPERTY, new PropertyChangeListener()
		{
			public void propertyChange(PropertyChangeEvent e)
			{
				updateChoosers((File)e.getNewValue());
			} // propertyChange
		}); // anonymous PropertyChangeListener
	} // initChoosers
	
	private void updateChoosers(File newdir)
	{
		configChooser.setCurrentDirectory(newdir);
		classChooser.setCurrentDirectory(newdir);
		dirChooser.setCurrentDirectory(newdir);
	} // updateChoosers
	
	/**
	 * Helper method to assist with closing the program
	 */
	private void shutdown()
	{
		String propname = System.getProperty("user.home") + File.separator + ".gretelrc";
		
		Properties p = new Properties();
		try
		{
			InputStream is = new FileInputStream(propname);
			p.load(is);
			
			is.close();
		}
		catch (IOException e) {}
		
		Rectangle r = getBounds();
		
		p.setProperty(TOP_BOUND_PROPERTY, String.valueOf(r.y));
		p.setProperty(LEFT_BOUND_PROPERTY, String.valueOf(r.x));
		p.setProperty(WIDTH_PROPERTY, String.valueOf(r.width));
		p.setProperty(HEIGHT_PROPERTY, String.valueOf(r.height));
		
		p.setProperty(CURRENT_TAB_PROPERTY,
			String.valueOf(jTabbedPane1.getSelectedIndex()));
		
		p.setProperty(CURRENT_CONFIG_PROPERTY, reinsConfigField.getText());
		
		try
		{
			OutputStream os = new FileOutputStream(propname);
			p.store(os, "Gretel configuration file.");
			os.close();
		}
		catch (IOException e) {}
		
		ListIterator li = viewers.listIterator(viewers.size());
		while (li.hasPrevious())
		{
			SourceViewer sv = (SourceViewer)li.previous();
			
			if (!sv.isStored())
			{
				sv.store();
				break;
			}
		} // while
		
		System.exit(0);
	} // shutdown
	
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jMenuBar3 = new javax.swing.JMenuBar();
		jMenu3 = new javax.swing.JMenu();
		quitItem = new javax.swing.JMenuItem();
		jMenu1 = new javax.swing.JMenu();
		insOptItem = new javax.swing.JMenuItem();
		jMenu4 = new javax.swing.JMenu();
		helpItem = new javax.swing.JMenuItem();
		jTabbedPane1 = new javax.swing.JTabbedPane();
		insPanel = new javax.swing.JPanel();
		jPanel5 = new javax.swing.JPanel();
		jScrollPane1 = new javax.swing.JScrollPane();
		insList = new javax.swing.JList();
		jPanel13 = new javax.swing.JPanel();
		insAddButton = new javax.swing.JButton();
		insRemoveButton = new javax.swing.JButton();
		insClearButton = new javax.swing.JButton();
		jPanel14 = new javax.swing.JPanel();
		jLabel1 = new javax.swing.JLabel();
		jPanel11 = new javax.swing.JPanel();
		jPanel8 = new javax.swing.JPanel();
		jLabel2 = new javax.swing.JLabel();
		jPanel15 = new javax.swing.JPanel();
		insDirectoryField = new javax.swing.JTextField();
		insBrowseButton = new javax.swing.JButton();
		jPanel16 = new javax.swing.JPanel();
		jPanel19 = new javax.swing.JPanel();
		jLabel5 = new javax.swing.JLabel();
		jPanel20 = new javax.swing.JPanel();
		insButton = new javax.swing.JButton();
		reinsPanel = new javax.swing.JPanel();
		jPanel23 = new javax.swing.JPanel();
		jPanel30 = new javax.swing.JPanel();
		jPanel31 = new javax.swing.JPanel();
		jLabel8 = new javax.swing.JLabel();
		jPanel32 = new javax.swing.JPanel();
		reinsConfigField = new javax.swing.JTextField();
		reinsBrowseButton = new javax.swing.JButton();
		jPanel28 = new javax.swing.JPanel();
		jScrollPane2 = new javax.swing.JScrollPane();
		reinsList = new javax.swing.JList();
		jPanel29 = new javax.swing.JPanel();
		jLabel9 = new javax.swing.JLabel();
		jPanel3 = new javax.swing.JPanel();
		viewButton = new javax.swing.JButton();
		jPanel25 = new javax.swing.JPanel();
		jPanel26 = new javax.swing.JPanel();
		jLabel10 = new javax.swing.JLabel();
		jPanel27 = new javax.swing.JPanel();
		reinsButton = new javax.swing.JButton();
		remPanel = new javax.swing.JPanel();
		jPanel34 = new javax.swing.JPanel();
		jScrollPane3 = new javax.swing.JScrollPane();
		remList = new javax.swing.JList();
		jPanel35 = new javax.swing.JPanel();
		remAddButton = new javax.swing.JButton();
		remRemoveButton = new javax.swing.JButton();
		remClearButton = new javax.swing.JButton();
		jPanel37 = new javax.swing.JPanel();
		jLabel11 = new javax.swing.JLabel();
		jPanel38 = new javax.swing.JPanel();
		jPanel39 = new javax.swing.JPanel();
		jLabel12 = new javax.swing.JLabel();
		jPanel40 = new javax.swing.JPanel();
		remButton = new javax.swing.JButton();
		
		jMenu3.setText("File");
		quitItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, java.awt.event.InputEvent.CTRL_MASK));
		quitItem.setText("Quit");
		quitItem.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				quitItemActionPerformed(evt);
			}
		});
		
		jMenu3.add(quitItem);
		jMenuBar3.add(jMenu3);
		jMenu1.setText("Settings");
		insOptItem.setText("Instrumentation Options");
		insOptItem.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				insOptItemActionPerformed(evt);
			}
		});
		
		jMenu1.add(insOptItem);
		jMenuBar3.add(jMenu1);
		jMenu4.setText("Help");
		helpItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0));
		helpItem.setText("Help");
		helpItem.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				helpItemActionPerformed(evt);
			}
		});
		
		jMenu4.add(helpItem);
		jMenuBar3.add(jMenu4);
		
		setTitle("Gretel");
		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				windowClosedAction(evt);
			}
		});
		
		insPanel.setLayout(new javax.swing.BoxLayout(insPanel, javax.swing.BoxLayout.Y_AXIS));
		
		jPanel5.setLayout(new java.awt.BorderLayout());
		
		jPanel5.setBorder(new javax.swing.border.EtchedBorder());
		jScrollPane1.setViewportView(insList);
		
		jPanel5.add(jScrollPane1, java.awt.BorderLayout.CENTER);
		
		jPanel13.setLayout(new ExpansionLayout());
		insAddButton.setText("Add files");
		insAddButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				insAddButtonActionPerformed(evt);
			}
		});
		
		jPanel13.add(insAddButton);
		
		insRemoveButton.setText("Remove Selected");
		insRemoveButton.setEnabled(false);
		insRemoveButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				insRemoveButtonActionPerformed(evt);
			}
		});
		
		jPanel13.add(insRemoveButton);
		
		insClearButton.setText("Clear List");
		insClearButton.setEnabled(false);
		insClearButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				insClearButtonActionPerformed(evt);
			}
		});
		
		jPanel13.add(insClearButton);
		
		jPanel5.add(jPanel13, java.awt.BorderLayout.SOUTH);
		
		jPanel14.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		
		jLabel1.setText("Step 1 - Choose files to instrument:");
		jPanel14.add(jLabel1);
		
		jPanel5.add(jPanel14, java.awt.BorderLayout.NORTH);
		
		insPanel.add(jPanel5);
		
		jPanel11.setLayout(new javax.swing.BoxLayout(jPanel11, javax.swing.BoxLayout.Y_AXIS));
		
		jPanel11.setBorder(new javax.swing.border.EtchedBorder());
		jPanel8.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		
		jLabel2.setText("Step 2 - Specify directory to store auxillary tables in:");
		jPanel8.add(jLabel2);
		
		jPanel11.add(jPanel8);
		
		jPanel15.setLayout(new ExpansionLayout());
		jPanel15.add(insDirectoryField);
		
		insBrowseButton.setText("Browse");
		insBrowseButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				insBrowseButtonActionPerformed(evt);
			}
		});
		
		jPanel15.add(insBrowseButton);
		
		jPanel11.add(jPanel15);
		
		insPanel.add(jPanel11);
		
		jPanel16.setLayout(new javax.swing.BoxLayout(jPanel16, javax.swing.BoxLayout.X_AXIS));
		
		jPanel16.setBorder(new javax.swing.border.EtchedBorder());
		jPanel19.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		
		jLabel5.setText("Step 3 - Click Here:");
		jPanel19.add(jLabel5);
		
		jPanel16.add(jPanel19);
		
		jPanel20.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
		
		insButton.setToolTipText(ins0);
		insButton.setText("Instrument Selected Files");
		insButton.setEnabled(false);
		insButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				insButtonActionPerformed(evt);
			}
		});
		
		jPanel20.add(insButton);
		
		jPanel16.add(jPanel20);
		
		insPanel.add(jPanel16);
		
		jTabbedPane1.addTab("Instrument", insPanel);
		
		reinsPanel.setLayout(new javax.swing.BoxLayout(reinsPanel, javax.swing.BoxLayout.Y_AXIS));
		
		jPanel23.setLayout(new javax.swing.BoxLayout(jPanel23, javax.swing.BoxLayout.Y_AXIS));
		
		jPanel23.setBorder(new javax.swing.border.EtchedBorder());
		jPanel30.setLayout(new javax.swing.BoxLayout(jPanel30, javax.swing.BoxLayout.Y_AXIS));
		
		jPanel31.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		
		jLabel8.setText("Step 1 - Locate the appropriate configuration file:");
		jPanel31.add(jLabel8);
		
		jPanel30.add(jPanel31);
		
		jPanel32.setLayout(new ExpansionLayout());
		jPanel32.add(reinsConfigField);
		
		reinsBrowseButton.setText("Browse");
		reinsBrowseButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				reinsBrowseButtonActionPerformed(evt);
			}
		});
		
		jPanel32.add(reinsBrowseButton);
		
		jPanel30.add(jPanel32);
		
		jPanel23.add(jPanel30);
		
		jPanel28.setLayout(new java.awt.BorderLayout());
		
		reinsList.addMouseListener(new java.awt.event.MouseAdapter()
		{
			public void mouseClicked(java.awt.event.MouseEvent evt)
			{
				reinsListMouseClicked(evt);
			}
		});
		
		jScrollPane2.setViewportView(reinsList);
		
		jPanel28.add(jScrollPane2, java.awt.BorderLayout.CENTER);
		
		jPanel29.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		
		jLabel9.setText("The selected configuration file is associated with these files:");
		jPanel29.add(jLabel9);
		
		jPanel28.add(jPanel29, java.awt.BorderLayout.NORTH);
		
		viewButton.setText("View Source of Selected File");
		viewButton.setEnabled(false);
		viewButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				viewButtonActionPerformed(evt);
			}
		});
		
		jPanel3.add(viewButton);
		
		jPanel28.add(jPanel3, java.awt.BorderLayout.SOUTH);
		
		jPanel23.add(jPanel28);
		
		reinsPanel.add(jPanel23);
		
		jPanel25.setLayout(new javax.swing.BoxLayout(jPanel25, javax.swing.BoxLayout.X_AXIS));
		
		jPanel25.setBorder(new javax.swing.border.EtchedBorder());
		jPanel26.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		
		jLabel10.setText("Step 2 - Click Here:");
		jPanel26.add(jLabel10);
		
		jPanel25.add(jPanel26);
		
		jPanel27.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
		
		reinsButton.setToolTipText(reins0);
		reinsButton.setText("Reinstrument Files");
		reinsButton.setEnabled(false);
		reinsButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				reinsButtonActionPerformed(evt);
			}
		});
		
		jPanel27.add(reinsButton);
		
		jPanel25.add(jPanel27);
		
		reinsPanel.add(jPanel25);
		
		jTabbedPane1.addTab("Reinstrument/View", reinsPanel);
		
		remPanel.setLayout(new javax.swing.BoxLayout(remPanel, javax.swing.BoxLayout.Y_AXIS));
		
		jPanel34.setLayout(new java.awt.BorderLayout());
		
		jPanel34.setBorder(new javax.swing.border.EtchedBorder());
		jScrollPane3.setViewportView(remList);
		
		jPanel34.add(jScrollPane3, java.awt.BorderLayout.CENTER);
		
		jPanel35.setLayout(new ExpansionLayout());
		
		remAddButton.setText("Add files");
		remAddButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				remAddButtonActionPerformed(evt);
			}
		});
		
		jPanel35.add(remAddButton);
		
		remRemoveButton.setText("Remove Selected");
		remRemoveButton.setEnabled(false);
		remRemoveButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				remRemoveButtonActionPerformed(evt);
			}
		});
		
		jPanel35.add(remRemoveButton);
		
		remClearButton.setText("Clear List");
		remClearButton.setEnabled(false);
		remClearButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				remClearButtonActionPerformed(evt);
			}
		});
		
		jPanel35.add(remClearButton);
		
		jPanel34.add(jPanel35, java.awt.BorderLayout.SOUTH);
		
		jPanel37.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		
		jLabel11.setText("Step 1 - Choose files to remove all instrumentation from:");
		jPanel37.add(jLabel11);
		
		jPanel34.add(jPanel37, java.awt.BorderLayout.NORTH);
		
		remPanel.add(jPanel34);
		
		jPanel38.setLayout(new javax.swing.BoxLayout(jPanel38, javax.swing.BoxLayout.X_AXIS));
		
		jPanel38.setBorder(new javax.swing.border.EtchedBorder());
		jPanel39.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
		
		jLabel12.setText("Step 2 - Click Here:");
		jPanel39.add(jLabel12);
		
		jPanel38.add(jPanel39);
		
		jPanel40.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
		
		remButton.setToolTipText(rem0);
		remButton.setText("Remove Instrumentation");
		remButton.setEnabled(false);
		remButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				remButtonActionPerformed(evt);
			}
		});
		
		jPanel40.add(remButton);
		
		jPanel38.add(jPanel40);
		
		remPanel.add(jPanel38);
		
		jTabbedPane1.addTab("Remove Instrumentation", remPanel);
		
		getContentPane().add(jTabbedPane1, java.awt.BorderLayout.CENTER);
		
		setJMenuBar(jMenuBar3);
		pack();
	}//GEN-END:initComponents

	private void windowClosedAction(java.awt.event.WindowEvent evt)//GEN-FIRST:event_windowClosedAction
	{//GEN-HEADEREND:event_windowClosedAction
		shutdown();
	}//GEN-LAST:event_windowClosedAction

	private void viewButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_viewButtonActionPerformed
	{//GEN-HEADEREND:event_viewButtonActionPerformed
		Object key = reinsList.getSelectedValue();
		
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try
		{
			SourceViewer sv = new SourceViewer((String)sources.get(key), false);
			sv.show();
			sv.fixBounds();
			viewers.add(sv);
		}
		catch (IOException e) {}
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}//GEN-LAST:event_viewButtonActionPerformed

	private void reinsListMouseClicked(java.awt.event.MouseEvent evt)//GEN-FIRST:event_reinsListMouseClicked
	{//GEN-HEADEREND:event_reinsListMouseClicked
		if (evt.getClickCount() != 2) return;
		int index = reinsList.locationToIndex(evt.getPoint());
		if (index == -1) return;
		
		String key = (String)reinsModel.get(index);
		
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try
		{
			SourceViewer sv = new SourceViewer((String)sources.get(key), false);
			sv.show();
			sv.fixBounds();
			viewers.add(sv);
		}
		catch (IOException ioe) {}
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}//GEN-LAST:event_reinsListMouseClicked
	
	/** 
	 * Method called when the "Options" menu item
	 * is activated.
	 */
	private void insOptItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_insOptItemActionPerformed
	{//GEN-HEADEREND:event_insOptItemActionPerformed
		Box b = new Box(BoxLayout.Y_AXIS);
		
		ButtonGroup bg = new ButtonGroup();
		
		JRadioButton b1 = new JRadioButton("No node reductions");
		JRadioButton b2 = new JRadioButton("Conservative node reductions");
		JRadioButton b3 = new JRadioButton("Aggressive node reductions (Default)");
		
		bg.add(b1);
		bg.add(b2);
		bg.add(b3);
		
		b.add(b1);
		b.add(b2);
		b.add(b3);
		
		switch (currentInsSetting)
		{
			case NOREDUCE:
				b1.setSelected(true);
				break;
			case CONSERVATIVE:
				b2.setSelected(true);
				break;
			case AGGRESSIVE:
				b3.setSelected(true);
				break;
			default:
				// Shouldn't be here.  Complain loudly.
				throw new IllegalStateException("Not a valid instrumentation setting.");
		} // switch
		
		int result = JOptionPane.showConfirmDialog(this, b, "Select setting", JOptionPane.OK_CANCEL_OPTION);
		
		if (result != JOptionPane.OK_OPTION) return;
		
		if (b1.isSelected())
			currentInsSetting = NOREDUCE;
		
		if (b2.isSelected())
			currentInsSetting = CONSERVATIVE;
		
		if (b3.isSelected())
			currentInsSetting = AGGRESSIVE;
	}//GEN-LAST:event_insOptItemActionPerformed
	
	/**
	 * This method is called when the Remove Instrumentation
	 * button in the remover panel is clicked.
	 */
	private void remButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_remButtonActionPerformed
	{//GEN-HEADEREND:event_remButtonActionPerformed
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		final ProgressDialog pd = new ProgressDialog(this, "Removing...");
		pd.showAsync();
		pd.waitUntilVisible();
		
		final Remover rem;
		try
		{
			rem = new Remover();
		}
		catch (NoClassDefFoundError e)
		{
			pd.dispose();
			setCursor(Cursor.getDefaultCursor());
			JOptionPane.showMessageDialog(this, "Unable to locate BCEL library.\nPlease make sure it is in the classpath.", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		rem.addInstrumentationEventListener(new InstrumentationEventListener()
		{
			private int max;
			
			public void instrumentationStarted(InstrumentationEvent e)
			{
				max = e.getClassesRemaining();
				pd.setMaximum(max);
			} // instrumentationStarted
			
			public void instrumentationFinished(InstrumentationEvent e) {}
			
			public void classStarted(InstrumentationEvent e)
			{
				pd.setValue(max - e.getClassesRemaining());
				pd.repaint();
			} // classStarted
			
		}); // anonymous InstrumentationEventListener
		
		// The rest of the computations should be done
		// outside the event-dispatching thread
		new Thread(new Runnable()
		{
			public void run()
			{
				try
				{
					Set remSet = new HashSet();
					
					for (int i = 0; i < remModel.size(); i++)
						remSet.add(remModel.getElementAt(i));
					
					rem.remove(remSet);
					pd.dispose();
					JOptionPane.showMessageDialog(Main.this, "Probes Removed", "Complete", JOptionPane.INFORMATION_MESSAGE);
				}
				catch (IOException e)
				{
					pd.dispose();
					JOptionPane.showMessageDialog(Main.this, "IO Error while removing probes", "Error", JOptionPane.ERROR_MESSAGE);
				}
				
				reinsConfigField.setText("");
				
				setCursor(Cursor.getDefaultCursor());
			} // run
		}).start(); // begin anonymous runnable
	}//GEN-LAST:event_remButtonActionPerformed
	
	/**
	 * Called when the clear button on the remover panel is clicked.
	 */
	private void remClearButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_remClearButtonActionPerformed
	{//GEN-HEADEREND:event_remClearButtonActionPerformed
		remModel.clear();
	}//GEN-LAST:event_remClearButtonActionPerformed
	
	/**
	 * Called when the remove selected button on the remover panel
	 * is clicked.
	 */
	private void remRemoveButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_remRemoveButtonActionPerformed
	{//GEN-HEADEREND:event_remRemoveButtonActionPerformed
		int [] toRemove = remList.getSelectedIndices();
		
		// it's in increasing order, so remove from the back
		// to front so that indices are still correct when an
		// item is removed
		for (int i = toRemove.length - 1; i >= 0; i--)
			remModel.removeElementAt(toRemove[i]);
	}//GEN-LAST:event_remRemoveButtonActionPerformed
	
	/**
	 * Called when the Add Files button on the remover panel
	 * is clicked.
	 */
	private void remAddButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_remAddButtonActionPerformed
	{//GEN-HEADEREND:event_remAddButtonActionPerformed
		int result = classChooser.showDialog(this, "Select Files");
		if (result != JFileChooser.APPROVE_OPTION) return;
		
		File [] selected = selected = classChooser.getSelectedFiles();
		for (int i = 0; i < selected.length; i++)
			remModel.addElement(selected[i]);
	}//GEN-LAST:event_remAddButtonActionPerformed
	
	/**
	 * Called when the Reinstrument button is clicked.
	 */
	private void reinsButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_reinsButtonActionPerformed
	{//GEN-HEADEREND:event_reinsButtonActionPerformed
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		final ProgressDialog pd = new ProgressDialog(this, "Reinstrumenting...");
		pd.showAsync();
		pd.waitUntilVisible();
		
		final Reinstrumenter reins;
		try
		{
			reins = new Reinstrumenter();
		}
		catch (NoClassDefFoundError e)
		{
			pd.dispose();
			setCursor(Cursor.getDefaultCursor());
			JOptionPane.showMessageDialog(this, "Unable to locate BCEL library.\nPlease make sure it is in the classpath.", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		reins.addInstrumentationEventListener(new InstrumentationEventListener()
		{
			private int max;
			
			public void instrumentationStarted(InstrumentationEvent e)
			{
				max = e.getClassesRemaining();
				pd.setMaximum(max);
			} // instrumentationStarted
			
			public void instrumentationFinished(InstrumentationEvent e) {}
			
			public void classStarted(InstrumentationEvent e)
			{
				pd.setValue(max - e.getClassesRemaining());
				pd.repaint();
			} // classStarted
			
		}); // anonymous InstrumentationEventListener
		
		// start a new thread for this so that the repaints will be visible
		new Thread(new Runnable()
		{
			public void run()
			{
				try
				{
					reins.reinstrument(reinsConfigField.getText());
					pd.dispose();
					JOptionPane.showMessageDialog(Main.this, "Reinstrumentation Complete", "Complete", JOptionPane.INFORMATION_MESSAGE);
				}
				catch (IOException e)
				{
					pd.dispose();
					JOptionPane.showMessageDialog(Main.this, "IO Error attempting to reinstrument.", "Error", JOptionPane.ERROR_MESSAGE);
					e.printStackTrace();
				}
				
				SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						updateReinsList();
						setCursor(Cursor.getDefaultCursor());
					} // run
				}); // anonymous runnable
			} // run
		}).start(); // begin new anonymous runnable
		
	}//GEN-LAST:event_reinsButtonActionPerformed
	
	/**
	 * Helper method to localize the logic for updating the
	 * contents of the JList listing instrumented files.
	 *
	 * Also enables/disables the Reinstrument button.
	 */
	private void updateReinsList()
	{
		reinsButton.setToolTipText(reins0);
		viewButton.setEnabled(false);
		viewButton.repaint();
		
		reinsConfig = reinsConfigField.getText();
		
		sources.clear();
		reinsModel.clear();
		
		try
		{
			File cfg = new File(reinsConfig);
			if (!cfg.exists() || !reinsConfig.endsWith(".gretel"))
			{
				reinsButton.setEnabled(false);
				reinsButton.repaint();
				return;
			}
		}
		catch (NullPointerException e)
		{
			reinsButton.setEnabled(false);
			reinsButton.repaint();
			return;
		}
		
		// First, get all the data from the tables
		Set s = new TreeSet();
		try
		{
			ListHits.setConfigFile(reinsConfig);
			ListHits.update();
			
			Table table = Table.getTable(Monitor.getLineTableName());
			
			Iterator it = table.iterator();
			while (it.hasNext())
			{
				TableEntry te = (TableEntry)it.next();
				
				sources.put(te.getClassName(), te.getSourceName());
				s.add(te.getClassName());
			} // while
		}
		catch (Exception e)
		{
			reinsButton.setEnabled(false);
			reinsButton.repaint();
			return;
		}
		
		Iterator it = s.iterator();
		while (it.hasNext())
			reinsModel.addElement(it.next());
		
		reinsButton.setToolTipText(reins1);
		
		reinsButton.setEnabled(true);
		reinsButton.repaint();
	} // updateReinsList
	
	/**
	 * Called when the browse button on the Reinstrument panel
	 * is clicked.
	 */	
	private void reinsBrowseButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_reinsBrowseButtonActionPerformed
	{//GEN-HEADEREND:event_reinsBrowseButtonActionPerformed
		int result = configChooser.showDialog(this, "Select Configuration");
		if (result != JFileChooser.APPROVE_OPTION) return;
		
		File f = configChooser.getSelectedFile();
		
		reinsConfigField.setText(f.toString());
		updateReinsList();
	}//GEN-LAST:event_reinsBrowseButtonActionPerformed
	
	/**
	 * Called when the Instrument button is clicked.
	 */
	private void insButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_insButtonActionPerformed
	{//GEN-HEADEREND:event_insButtonActionPerformed
		final Set s = new HashSet();
		
		for (int i = 0; i < insModel.size(); i++)
			s.add(insModel.get(i).toString());
		
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		final ProgressDialog pd = new ProgressDialog(this, "Instrumenting...");
		pd.showAsync();
		pd.waitUntilVisible();
		
		final Instrumenter ins;
		try
		{
			ins = new Instrumenter();
		}
		catch (NoClassDefFoundError e)
		{
			pd.dispose();
			setCursor(Cursor.getDefaultCursor());
			JOptionPane.showMessageDialog(this, "Unable to locate BCEL library.\nPlease make sure it is in the classpath.", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		switch (currentInsSetting)
		{
			case NOREDUCE:
				ins.setReduceNodes(false);
				break;
			case CONSERVATIVE:
				ins.setReduceNodes(true);
				ins.setBreakLoops(true);
				break;
			case AGGRESSIVE:
				ins.setReduceNodes(true);
				ins.setBreakLoops(false);
				break;
			default:
				// Shouldn't be here.  Complain loudly.
				throw new IllegalStateException("Not a valid instrumentation setting.");
		} // switch
		
		ins.addInstrumentationEventListener(new InstrumentationEventListener()
		{
			private int max;
			
			public void instrumentationStarted(InstrumentationEvent e)
			{
				max = e.getClassesRemaining();
				pd.setMaximum(max);
			} // instrumentationStarted
			
			public void instrumentationFinished(InstrumentationEvent e) {}
			
			public void classStarted(InstrumentationEvent e)
			{
				pd.setValue(max - e.getClassesRemaining());
				pd.repaint();
			} // classStarted
			
		}); // anonymous InstrumentationEventListener
		
		// start a new thread for this so that the repaints will be visible
		new Thread(new Runnable()
		{
			public void run()
			{
				String tmp = null;
				try
				{
					File dir = new File(insDirectoryField.getText());
					
					String dirfield = dir.getAbsolutePath().concat(File.separator);
					
					ins.instrument(s, dirfield);
					pd.dispose();
					Set names = ins.getConfigNames();
					if (names.size() > 0)
					{
						JOptionPane.showMessageDialog(Main.this, "Instrumentation Complete", "Complete", JOptionPane.INFORMATION_MESSAGE);
						tmp = ((String)names.iterator().next()).concat(".gretel");
					}
				}
				catch (IOException ioe)
				{
					pd.dispose();
					JOptionPane.showMessageDialog(Main.this, "IO error during instrumentation", "Error", JOptionPane.ERROR_MESSAGE);
				}
				
				final String cfg = tmp;
				
				if (cfg != null)
				{
					SwingUtilities.invokeLater(new Runnable()
					{
						public void run()
						{
							reinsConfigField.setText(cfg);
							jTabbedPane1.setSelectedComponent(reinsPanel);
						} // run
					}); // anonymous Runnable
				} // if
				
				setCursor(Cursor.getDefaultCursor());
			} // run
		}).start(); // starting anonymous Runnable
	}//GEN-LAST:event_insButtonActionPerformed
	
	/**
	 * Called when the browse button on the Instrument panel
	 * is clicked.
	 */
	private void insBrowseButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_insBrowseButtonActionPerformed
	{//GEN-HEADEREND:event_insBrowseButtonActionPerformed
		int result = dirChooser.showDialog(this, "Select Directory");
		if (result != JFileChooser.APPROVE_OPTION) return;
		
		File f = dirChooser.getSelectedFile();
		insDirectoryField.setText(f.toString());
	}//GEN-LAST:event_insBrowseButtonActionPerformed
	
	/**
	 * Called when the clear button on the Instrument panel
	 * is clicked.
	 */
	private void insClearButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_insClearButtonActionPerformed
	{//GEN-HEADEREND:event_insClearButtonActionPerformed
		insModel.clear();
	}//GEN-LAST:event_insClearButtonActionPerformed
	
	/**
	 * Called when the Remove Selected button on the Instrument
	 * panel is clicked.
	 */
	private void insRemoveButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_insRemoveButtonActionPerformed
	{//GEN-HEADEREND:event_insRemoveButtonActionPerformed
		int [] toRemove = insList.getSelectedIndices();
		
		// it's in increasing order, so remove from the back
		// to front so that indices are still correct when an
		// item is removed
		for (int i = toRemove.length - 1; i >= 0; i--)
			insModel.removeElementAt(toRemove[i]);
	}//GEN-LAST:event_insRemoveButtonActionPerformed
	
	/**
	 * Called when the Add Files button in the Instrument panel
	 * is clicked.
	 */
	private void insAddButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_insAddButtonActionPerformed
	{//GEN-HEADEREND:event_insAddButtonActionPerformed
		int result = classChooser.showDialog(this, "Select Files");
		if (result != JFileChooser.APPROVE_OPTION) return;
		
		File [] selected = selected = classChooser.getSelectedFiles();
		for (int i = 0; i < selected.length; i++)
			insModel.addElement(selected[i]);
	}//GEN-LAST:event_insAddButtonActionPerformed
	
	/**
	 * Called when the Help menu item is clicked on.
	 */
	private void helpItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_helpItemActionPerformed
	{//GEN-HEADEREND:event_helpItemActionPerformed
		if (hf == null)
		{
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			try
			{
				hf = new HelpFrame(ClassLoader.getSystemResource("docs/help.html"));
			}
			catch (IOException e) {}
			catch (NullPointerException e) {}
			setCursor(Cursor.getDefaultCursor());
		}
		
		try
		{
			hf.show();
		}
		catch (NullPointerException e)
		{
			JOptionPane.showMessageDialog(this, "Unable to load help file", "Error", JOptionPane.ERROR_MESSAGE);
		}
	}//GEN-LAST:event_helpItemActionPerformed
	
	/**
	 * Called when the quit menu item is selected.
	 */
	private void quitItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_quitItemActionPerformed
	{//GEN-HEADEREND:event_quitItemActionPerformed
		shutdown();
	}//GEN-LAST:event_quitItemActionPerformed
	
	/**
	 * Main to either start GUI or dispatch to one of the other
	 * tools.
	 */
	public static void main(String args[]) throws Exception
	{
		if (args.length == 0)
		{
			Main m = new Main();
			m.show();
			m.setBounds(m.correctBounds);
		}
		else
		{
			String [] arg2 = new String[args.length - 1];
			
			for (int i = 0; i < arg2.length; i++)
				arg2[i] = args[i + 1];
			
			if (args[0].regionMatches(true, 0, "in", 0, 2))
			{
				Instrumenter.main(arg2);
			}
			else if (args[0].regionMatches(true, 0, "rem", 0, 3))
			{
				Remover.main(arg2);
			}
			else if (args[0].regionMatches(true, 0, "re", 0, 2))
			{
				Reinstrumenter.main(arg2);
			}
			else if (args[0].regionMatches(true, 0, "li", 0, 2))
			{
				ListHits.main(arg2);
			}
			else if (args[0].regionMatches(true, 0, "so", 0, 2))
			{
				SourceViewer.main(arg2);
			}
			else
			{
				System.out.println("Not a recognized option.  Running with no arguments");
				System.out.println("is recommended.");
			}
		}
	} // main
	
	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JMenuBar jMenuBar3;
	private javax.swing.JMenu jMenu3;
	private javax.swing.JMenuItem quitItem;
	private javax.swing.JMenu jMenu1;
	private javax.swing.JMenuItem insOptItem;
	private javax.swing.JMenu jMenu4;
	private javax.swing.JMenuItem helpItem;
	private javax.swing.JTabbedPane jTabbedPane1;
	private javax.swing.JPanel insPanel;
	private javax.swing.JPanel jPanel5;
	private javax.swing.JScrollPane jScrollPane1;
	private javax.swing.JList insList;
	private javax.swing.JPanel jPanel13;
	private javax.swing.JButton insAddButton;
	private javax.swing.JButton insRemoveButton;
	private javax.swing.JButton insClearButton;
	private javax.swing.JPanel jPanel14;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JPanel jPanel11;
	private javax.swing.JPanel jPanel8;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JPanel jPanel15;
	private javax.swing.JTextField insDirectoryField;
	private javax.swing.JButton insBrowseButton;
	private javax.swing.JPanel jPanel16;
	private javax.swing.JPanel jPanel19;
	private javax.swing.JLabel jLabel5;
	private javax.swing.JPanel jPanel20;
	private javax.swing.JButton insButton;
	private javax.swing.JPanel reinsPanel;
	private javax.swing.JPanel jPanel23;
	private javax.swing.JPanel jPanel30;
	private javax.swing.JPanel jPanel31;
	private javax.swing.JLabel jLabel8;
	private javax.swing.JPanel jPanel32;
	private javax.swing.JTextField reinsConfigField;
	private javax.swing.JButton reinsBrowseButton;
	private javax.swing.JPanel jPanel28;
	private javax.swing.JScrollPane jScrollPane2;
	private javax.swing.JList reinsList;
	private javax.swing.JPanel jPanel29;
	private javax.swing.JLabel jLabel9;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JButton viewButton;
	private javax.swing.JPanel jPanel25;
	private javax.swing.JPanel jPanel26;
	private javax.swing.JLabel jLabel10;
	private javax.swing.JPanel jPanel27;
	private javax.swing.JButton reinsButton;
	private javax.swing.JPanel remPanel;
	private javax.swing.JPanel jPanel34;
	private javax.swing.JScrollPane jScrollPane3;
	private javax.swing.JList remList;
	private javax.swing.JPanel jPanel35;
	private javax.swing.JButton remAddButton;
	private javax.swing.JButton remRemoveButton;
	private javax.swing.JButton remClearButton;
	private javax.swing.JPanel jPanel37;
	private javax.swing.JLabel jLabel11;
	private javax.swing.JPanel jPanel38;
	private javax.swing.JPanel jPanel39;
	private javax.swing.JLabel jLabel12;
	private javax.swing.JPanel jPanel40;
	private javax.swing.JButton remButton;
	// End of variables declaration//GEN-END:variables
	
} // class Main
